@@ -76,12 +76,6 @@
 #include <rpc/xdr_ioq.h>
 #include "svc_ioq.h"
 
-#ifndef __APPLE__
-struct cmessage {
-	struct cmsghdr cmsg;
-	struct cmsgcred cmcred;
-};
-#endif
 
 static enum clnt_stat clnt_vc_call(CLIENT *, AUTH *, rpcproc_t, xdrproc_t,
 				   void *, xdrproc_t, void *, struct timeval);
@@ -241,13 +235,6 @@ clnt_vc_ncreate2(int fd,	/* open file descriptor */
 	}
 
 	clnt = (CLIENT *) mem_alloc(sizeof(CLIENT));
-	if (!clnt) {
-		(void)syslog(LOG_ERR, clnt_vc_errstr, clnt_vc_str,
-			     __no_mem_str);
-		rpc_createerr.cf_stat = RPC_SYSTEMERROR;
-		rpc_createerr.cf_error.re_errno = errno;
-		goto err;
-	}
 
 	mutex_init(&clnt->cl_lock, NULL);
 	clnt->cl_flags = CLNT_FLAG_NONE;
@@ -256,13 +243,13 @@ clnt_vc_ncreate2(int fd,	/* open file descriptor */
 	/* private data struct */
 	xd->cx.data.ct_fd = fd;
 	cs = mem_alloc(sizeof(struct ct_serialized));
+
 	ct = &xd->cx.data;
 	ct->ct_closeit = false;
 	ct->ct_wait.tv_usec = 0;
 	ct->ct_waitset = false;
 	ct->ct_addr.buf = mem_alloc(raddr->maxlen);
-	if (ct->ct_addr.buf == NULL)
-		goto err;
+
 	memcpy(ct->ct_addr.buf, raddr->buf, raddr->len);
 	ct->ct_addr.len = raddr->len;
 	ct->ct_addr.maxlen = raddr->maxlen;
@@ -306,23 +293,32 @@ clnt_vc_ncreate2(int fd,	/* open file descriptor */
 	return (clnt);
 
  err:
+	/* XXX fix */
+	if (cs) {
+		mem_free(cs, sizeof(struct ct_serialized));
+	}
+
 	if (clnt) {
-		/* XXX fix */
-		if (xd) {
-			if (ct->ct_addr.len)
-				mem_free(ct->ct_addr.buf, ct->ct_addr.len);
-			free_x_vc_data(xd);
-		}
-		if (cs) {
-			mem_free(cs, sizeof(struct ct_serialized));
-		}
 		mutex_destroy(&clnt->cl_lock);
 		mem_free(clnt, sizeof(CLIENT));
 	}
+
 	if (rec) {
 		if (rpc_dplx_unref(rec, RPC_DPLX_FLAG_LOCKED))
 			REC_UNLOCK(rec);
 	}
+
+	if (xd) {
+		if (ct->ct_addr.len)
+			mem_free(ct->ct_addr.buf, ct->ct_addr.len);
+
+		if (xd->refcnt == 0) {
+			XDR_DESTROY(&xd->shared.xdrs_in);
+			XDR_DESTROY(&xd->shared.xdrs_out);
+			free_x_vc_data(xd);
+		}
+	}
+
 	thr_sigsetmask(SIG_SETMASK, &(mask), NULL);
 	return (NULL);
 }
