@@ -51,6 +51,9 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <netdb.h>
+#ifdef RPC_VSOCK
+#include <linux/vm_sockets.h>
+#endif /* VSOCK */
 #include <netconfig.h>
 #include <err.h>
 #include <stdlib.h>
@@ -59,21 +62,71 @@
 
 #include "rpc_com.h"
 
+void
+thr_keyfree(void *k)
+{
+	mem_free(k, 0);	/* XXX */
+}
+
+static void
+tirpc_free(void *p, size_t unused)
+{
+	free(p);
+}
+
+static void *
+tirpc_malloc(size_t size, const char *file, int line, const char *function)
+{
+	return malloc(size);
+}
+
+static void *
+tirpc_aligned(size_t alignment, size_t size, const char *file, int line,
+	      const char *function)
+{
+#if defined(_ISOC11_SOURCE)
+	return aligned_alloc(alignment, size);
+#else
+	void *ptr;
+	(void) posix_memalign(&ptr, alignment, size);
+	return ptr;
+#endif
+}
+
+static void *
+tirpc_calloc(size_t count, size_t size, const char *file, int line,
+	     const char *function)
+{
+	return calloc(count, size);
+}
+
+static void *
+tirpc_realloc(void *p, size_t size, const char *file, int line,
+	      const char *function)
+{
+	return realloc(p, size);
+}
+
 tirpc_pkg_params __ntirpc_pkg_params = {
 	TIRPC_FLAG_NONE,
 	TIRPC_DEBUG_FLAG_NONE,
-	warnx
+	warnx,
+	tirpc_free,
+	tirpc_malloc,
+	tirpc_aligned,
+	tirpc_calloc,
+	tirpc_realloc,
 };
 
 bool
 tirpc_control(const u_int rq, void *in)
 {
 	switch (rq) {
-	case TIRPC_GET_FLAGS:
-		*(u_int *) in = __ntirpc_pkg_params.flags;
+	case TIRPC_GET_PARAMETERS:
+		*(tirpc_pkg_params *)in = __ntirpc_pkg_params;
 		break;
-	case TIRPC_SET_FLAGS:
-		__ntirpc_pkg_params.flags = *(int *)in;
+	case TIRPC_PUT_PARAMETERS:
+		__ntirpc_pkg_params = *(tirpc_pkg_params *)in;
 		break;
 	case TIRPC_GET_DEBUG_FLAGS:
 		*(u_int *) in = __ntirpc_pkg_params.debug_flags;
@@ -81,11 +134,11 @@ tirpc_control(const u_int rq, void *in)
 	case TIRPC_SET_DEBUG_FLAGS:
 		__ntirpc_pkg_params.debug_flags = *(int *)in;
 		break;
-	case TIRPC_GET_WARNX:
-		*(warnx_t *) in = __ntirpc_pkg_params.warnx;
+	case TIRPC_GET_OTHER_FLAGS:
+		*(u_int *) in = __ntirpc_pkg_params.other_flags;
 		break;
-	case TIRPC_SET_WARNX:
-		__ntirpc_pkg_params.warnx = *(warnx_t) in;
+	case TIRPC_SET_OTHER_FLAGS:
+		__ntirpc_pkg_params.other_flags = *(int *)in;
 		break;
 	default:
 		return (false);
@@ -99,20 +152,22 @@ struct handle {
 	int nettype;
 };
 
-static const struct _rpcnettype {
+struct _rpcnettype {
 	const char *name;
 	const int type;
-} _rpctypelist[] = {
-	{
-	"netpath", _RPC_NETPATH}, {
-	"visible", _RPC_VISIBLE}, {
-	"circuit_v", _RPC_CIRCUIT_V}, {
-	"datagram_v", _RPC_DATAGRAM_V}, {
-	"circuit_n", _RPC_CIRCUIT_N}, {
-	"datagram_n", _RPC_DATAGRAM_N}, {
-	"tcp", _RPC_TCP}, {
-	"udp", _RPC_UDP}, {
-	0, _RPC_NONE}
+};
+
+static const struct _rpcnettype _rpctypelist[] = {
+	{"netpath", _RPC_NETPATH},
+	{"visible", _RPC_VISIBLE},
+	{"circuit_v", _RPC_CIRCUIT_V},
+	{"datagram_v", _RPC_DATAGRAM_V},
+	{"circuit_n", _RPC_CIRCUIT_N},
+	{"datagram_n", _RPC_DATAGRAM_N},
+	{"tcp", _RPC_TCP},
+	{"udp", _RPC_UDP},
+	{"vsock", _RPC_VSOCK},
+	{0, _RPC_NONE}
 };
 
 struct netid_af {
@@ -128,6 +183,9 @@ static const struct netid_af na_cvt[] = {
 	{"udp6", AF_INET6, IPPROTO_UDP},
 	{"tcp6", AF_INET6, IPPROTO_TCP},
 #endif
+#ifdef RPC_VSOCK
+	{"vsock", AF_VSOCK, PF_VSOCK},
+#endif /* VSOCK */
 	{"local", AF_LOCAL, 0}
 };
 
@@ -173,8 +231,13 @@ __rpc_get_t_size(int af, int proto, int size)
 	case IPPROTO_UDP:
 		defsize = UDPMSGSIZE;
 		break;
+#ifdef RPC_VSOCK
+	case PF_VSOCK:
+		defsize = 64 * 1024;	/* XXX */
+		break;
+#endif /* VSOCK */
 	default:
-		defsize = RPC_MAXDATASIZE;
+		defsize = RPC_MAXDATASIZE; /* 9000 */
 		break;
 	}
 	if (size == 0)
@@ -197,6 +260,11 @@ __rpc_get_a_size(int af)
 	case AF_INET6:
 		return sizeof(struct sockaddr_in6);
 #endif
+#ifdef RPC_VSOCK
+		/* Linux */
+	case AF_VSOCK:
+		return sizeof(struct sockaddr_vm);
+#endif /* VSOCK */
 	case AF_LOCAL:
 		return sizeof(struct sockaddr_un);
 	default:
@@ -251,24 +319,32 @@ __rpc_getconfip(const char *nettype)
 	char *netid;
 	char *netid_tcp = (char *)NULL;
 	char *netid_udp = (char *)NULL;
+	char *netid_vsock = (char *)NULL;
 	struct netconfig *dummy;
-	extern thread_key_t tcp_key, udp_key;
+	extern thread_key_t tcp_key, udp_key, vsock_key;
 	extern mutex_t tsd_lock;
 
 	if (tcp_key == -1) {
 		mutex_lock(&tsd_lock);
 		if (tcp_key == -1)
-			thr_keycreate(&tcp_key, free);	/* XXX */
+			thr_keycreate(&tcp_key, thr_keyfree);
 		mutex_unlock(&tsd_lock);
 	}
 	netid_tcp = (char *)thr_getspecific(tcp_key);
 	if (udp_key == -1) {
 		mutex_lock(&tsd_lock);
 		if (udp_key == -1)
-			thr_keycreate(&udp_key, free);
+			thr_keycreate(&udp_key, thr_keyfree);
 		mutex_unlock(&tsd_lock);
 	}
 	netid_udp = (char *)thr_getspecific(udp_key);
+	if (vsock_key == -1) {
+		mutex_lock(&tsd_lock);
+		if (vsock_key == -1)
+			thr_keycreate(&vsock_key, free);
+		mutex_unlock(&tsd_lock);
+	}
+	netid_vsock = (char *)thr_getspecific(vsock_key);
 	if (!netid_udp && !netid_tcp) {
 		struct netconfig *nconf;
 		void *confighandle;
@@ -284,16 +360,24 @@ __rpc_getconfip(const char *nettype)
 			    || strcmp(nconf->nc_protofmly, NC_INET6) == 0) {
 				if (strcmp(nconf->nc_proto, NC_TCP) == 0
 				    && netid_tcp == NULL) {
-					netid_tcp = rpc_strdup(nconf->nc_netid);
+					netid_tcp = mem_strdup(nconf->nc_netid);
 					thr_setspecific(tcp_key,
 							(void *)netid_tcp);
 				} else if (strcmp(nconf->nc_proto, NC_UDP) == 0
 					   && netid_udp == NULL) {
-					netid_udp = rpc_strdup(nconf->nc_netid);
+					netid_udp = mem_strdup(nconf->nc_netid);
 					thr_setspecific(udp_key,
 							(void *)netid_udp);
 				}
-			}
+			} /* NC_INET || NC_INET6 */
+			if (strcmp(nconf->nc_protofmly, NC_VSOCK) == 0) {
+				if (netid_vsock == NULL) {
+					netid_vsock =
+						mem_strdup(nconf->nc_netid);
+					thr_setspecific(vsock_key,
+							(void *)netid_vsock);
+				}
+			} /* VSOCK */
 		}
 		endnetconfig(confighandle);
 	}
@@ -301,6 +385,8 @@ __rpc_getconfip(const char *nettype)
 		netid = netid_udp;
 	else if (strcmp(nettype, "tcp") == 0)
 		netid = netid_tcp;
+	else if (strcmp(nettype, "vsock") == 0)
+		netid = netid_vsock;
 	else
 		return (NULL);
 	if ((netid == NULL) || (netid[0] == 0))
@@ -317,18 +403,15 @@ __rpc_getconfip(const char *nettype)
 void *
 __rpc_setconf(const char *nettype)
 {
-	struct handle *handle;
+	struct handle *handle = (struct handle *)mem_zalloc(sizeof(*handle));
 
-	handle = (struct handle *)mem_zalloc(sizeof(struct handle));
-	if (handle == NULL)
-		return (NULL);
 	switch (handle->nettype = getnettype(nettype)) {
 	case _RPC_NETPATH:
 	case _RPC_CIRCUIT_N:
 	case _RPC_DATAGRAM_N:
 		handle->nhandle = setnetpath();
 		if (!handle->nhandle) {
-			mem_free(handle, 0);	/* XXX */
+			mem_free(handle, sizeof(*handle));
 			return (NULL);
 		}
 		handle->nflag = true;
@@ -338,17 +421,18 @@ __rpc_setconf(const char *nettype)
 	case _RPC_DATAGRAM_V:
 	case _RPC_TCP:
 	case _RPC_UDP:
+	case _RPC_VSOCK:
 		handle->nhandle = setnetconfig();
 		if (!handle->nhandle) {
 			__warnx(TIRPC_DEBUG_FLAG_DEFAULT,
 				"rpc: failed to open %s", NETCONFIG);
-			mem_free(handle, 0);
+			mem_free(handle, sizeof(*handle));
 			return (NULL);
 		}
 		handle->nflag = false;
 		break;
 	default:
-		mem_free(handle, 0);
+		mem_free(handle, sizeof(*handle));
 		return (NULL);
 	}
 
@@ -362,10 +446,9 @@ __rpc_setconf(const char *nettype)
 struct netconfig *
 __rpc_getconf(void *vhandle)
 {
-	struct handle *handle;
 	struct netconfig *nconf;
+	struct handle *handle = (struct handle *)vhandle;
 
-	handle = (struct handle *)vhandle;
 	if (handle == NULL)
 		return (NULL);
 	for (;;) {
@@ -426,6 +509,14 @@ __rpc_getconf(void *vhandle)
 			    || strcmp(nconf->nc_proto, NC_UDP))
 				continue;
 			break;
+                case _RPC_VSOCK:
+			/* accept stream sockets only, there is no valid
+			 * proto (i.e., it must be "-") */
+			if ((nconf->nc_semantics != NC_TPI_COTS_ORD) ||
+				(strcmp(nconf->nc_protofmly, NC_VSOCK)) ||
+				(strcmp(nconf->nc_proto, NC_NOPROTO)))
+				continue;
+			break;
 		}
 		break;
 	}
@@ -435,16 +526,15 @@ __rpc_getconf(void *vhandle)
 void
 __rpc_endconf(void *vhandle)
 {
-	struct handle *handle;
+	struct handle *handle = (struct handle *)vhandle;
 
-	handle = (struct handle *)vhandle;
 	if (handle == NULL)
 		return;
 	if (handle->nflag)
 		endnetpath(handle->nhandle);
 	else
 		endnetconfig(handle->nhandle);
-	mem_free(handle, 0);
+	mem_free(handle, sizeof(*handle));
 }
 
 /*
@@ -491,7 +581,7 @@ int
 __rpc_fd2sockinfo(int fd, struct __rpc_sockinfo *sip)
 {
 	socklen_t len;
-	int type, proto;
+	int type, proto = 0;
 	struct sockaddr_storage ss;
 
 	len = sizeof(ss);
@@ -505,9 +595,19 @@ __rpc_fd2sockinfo(int fd, struct __rpc_sockinfo *sip)
 
 	/* XXX */
 	if (ss.ss_family != AF_LOCAL) {
-		if (type == SOCK_STREAM)
-			proto = IPPROTO_TCP;
-		else if (type == SOCK_DGRAM)
+		if (type == SOCK_STREAM) {
+			switch (ss.ss_family) {
+			case PF_INET:
+			case PF_INET6:
+				proto = IPPROTO_TCP;
+				break;
+#ifdef RPC_VSOCK
+			case AF_VSOCK:
+				proto = PF_VSOCK;
+				break;
+#endif /* VSOCK */
+			}
+		} else if (type == SOCK_DGRAM)
 			proto = IPPROTO_UDP;
 		else
 			return 0;
@@ -642,12 +742,13 @@ __rpc_taddr2uaddr_af(int af, const struct netbuf *nbuf)
 		goto out;
 
 	/* XXX check (fix) this */
-	ret = mem_zalloc(INET6_ADDRSTRLEN + (2 * 6) + 1);
-	if (!ret)
-		return (NULL);
+#define RETURN_SIZE (INET6_ADDRSTRLEN + (2 * 6) + 1)
+	ret = mem_zalloc(RETURN_SIZE);
 
 	switch (af) {
 	case AF_INET:
+		if (nbuf->len < sizeof(*sin))
+			return NULL;
 		sin = nbuf->buf;
 		if (inet_ntop(af, &sin->sin_addr, namebuf, sizeof(namebuf))
 		    == NULL)
@@ -656,27 +757,44 @@ __rpc_taddr2uaddr_af(int af, const struct netbuf *nbuf)
 		if (sprintf
 		    (ret, "%s.%u.%u", namebuf, ((u_int32_t) port) >> 8,
 		     port & 0xff) < 0) {
-			mem_free(ret, 0);
+			mem_free(ret, RETURN_SIZE);
 			return NULL;
 		}
 		break;
 #ifdef INET6
 	case AF_INET6:
+		if (nbuf->len < sizeof(*sin6))
+			return NULL;
 		sin6 = nbuf->buf;
 		if (inet_ntop(af, &sin6->sin6_addr, namebuf6, sizeof(namebuf6))
 		    == NULL) {
-			mem_free(ret, 0);
+			mem_free(ret, RETURN_SIZE);
 			return NULL;
 		}
 		port = ntohs(sin6->sin6_port);
-		if (sprintf
-		    (ret, "%s.%u.%u", namebuf6, ((u_int32_t) port) >> 8,
-		     port & 0xff) < 0) {
-			mem_free(ret, 0);
+		if (sprintf(ret, "%s.%u.%u",
+				namebuf6,
+				((u_int32_t) port) >> 8,
+				port & 0xff) < 0) {
+			mem_free(ret, RETURN_SIZE);
 			return NULL;
 		}
 		break;
 #endif
+#ifdef RPC_VSOCK
+	case AF_VSOCK:
+	{
+		struct sockaddr_vm *svm = nbuf->buf;
+		port = svm->svm_port;
+		if (sprintf
+			(ret, "%d.%u.%u", svm->svm_cid, ((u_int32_t)port) >> 8,
+				port & 0xff) < 0) {
+			mem_free(ret, 0);
+			return NULL;
+		}
+	}
+	break;
+#endif /* VSOCK */
 	case AF_LOCAL:
 		sun = nbuf->buf;
 		/* if (asprintf(&ret, "%.*s", (int)(sun->sun_len -
@@ -686,12 +804,12 @@ __rpc_taddr2uaddr_af(int af, const struct netbuf *nbuf)
 			    (int)(sizeof(*sun) -
 				  offsetof(struct sockaddr_un, sun_path)),
 			    sun->sun_path) < 0) {
-			mem_free(ret, 0);
+			mem_free(ret, RETURN_SIZE);
 			return NULL;
 		}
 		break;
 	default:
-		mem_free(ret, 0);
+		mem_free(ret, RETURN_SIZE);
 		return NULL;
 	}
 
@@ -713,13 +831,13 @@ __rpc_uaddr2taddr_af(int af, const char *uaddr)
 
 	port = 0;
 	sin = NULL;
-	addrstr = rpc_strdup(uaddr);
-	if (addrstr == NULL)
+	if (uaddr == NULL)
 		return NULL;
+	addrstr = mem_strdup(uaddr);
 
 	/*
 	 * AF_LOCAL addresses are expected to be absolute
-	 * pathnames, anything else will be AF_INET or AF_INET6.
+	 * pathnames.
 	 */
 	if (*addrstr != '/') {
 		p = strrchr(addrstr, '.');
@@ -737,20 +855,16 @@ __rpc_uaddr2taddr_af(int af, const char *uaddr)
 	}
 
 	ret = (struct netbuf *)mem_zalloc(sizeof(*ret));
-	if (ret == NULL)
-		goto out;
 
 	switch (af) {
 	case AF_INET:
 		sin = (struct sockaddr_in *)mem_zalloc(sizeof(*sin));
-		if (sin == NULL)
-			goto out;
-		memset(sin, 0, sizeof(*sin));
+
 		sin->sin_family = AF_INET;
 		sin->sin_port = htons(port);
 		if (inet_pton(AF_INET, addrstr, &sin->sin_addr) <= 0) {
-			mem_free(sin, 0);	/* XXX */
-			mem_free(ret, 0);
+			mem_free(sin, sizeof(*sin));
+			mem_free(ret, sizeof(*ret));
 			ret = NULL;
 			goto out;
 		}
@@ -760,14 +874,12 @@ __rpc_uaddr2taddr_af(int af, const char *uaddr)
 #ifdef INET6
 	case AF_INET6:
 		sin6 = (struct sockaddr_in6 *)mem_zalloc(sizeof(*sin6));
-		if (sin6 == NULL)
-			goto out;
-		memset(sin6, 0, sizeof(*sin6));
+
 		sin6->sin6_family = AF_INET6;
 		sin6->sin6_port = htons(port);
 		if (inet_pton(AF_INET6, addrstr, &sin6->sin6_addr) <= 0) {
-			mem_free(sin6, 0);
-			mem_free(ret, 0);
+			mem_free(sin6, sizeof(*sin6));
+			mem_free(ret, sizeof(*ret));
 			ret = NULL;
 			goto out;
 		}
@@ -775,11 +887,35 @@ __rpc_uaddr2taddr_af(int af, const char *uaddr)
 		ret->buf = sin6;
 		break;
 #endif
+#ifdef RPC_VSOCK
+	case AF_VSOCK:
+	{
+		struct sockaddr_in sin;
+		struct sockaddr_vm *svm;
+		svm = (struct sockaddr_vm *) mem_zalloc(
+			sizeof(struct sockaddr_vm));
+		if (svm == NULL)
+			goto out;
+		svm->svm_family = AF_VSOCK;
+		memset(&sin, 0, sizeof(sin));
+		sin.sin_family = AF_INET;
+		sin.sin_port = htons(port);
+		if (inet_pton(AF_INET, addrstr, &sin.sin_addr) <= 0) {
+			mem_free(svm, 0);
+			mem_free(ret, 0);
+			ret = NULL;
+			goto out;
+		}
+		svm->svm_cid = sin.sin_addr.s_addr;
+		svm->svm_port = port;
+		ret->maxlen = ret->len = sizeof(struct sockaddr_vm);
+		ret->buf = svm;
+	}
+	break;
+#endif /* VSOCK */
 	case AF_LOCAL:
 		sun = (struct sockaddr_un *)mem_zalloc(sizeof(*sun));
-		if (sun == NULL)
-			goto out;
-		memset(sun, 0, sizeof(*sun));
+
 		sun->sun_family = AF_LOCAL;
 		strncpy(sun->sun_path, addrstr, sizeof(sun->sun_path) - 1);
 		ret->len = SUN_LEN(sun);
@@ -787,7 +923,7 @@ __rpc_uaddr2taddr_af(int af, const char *uaddr)
 		ret->buf = sun;
 		break;
 	default:
-		mem_free(ret, 0);
+		mem_free(ret, sizeof(*ret));
 		ret = NULL;
 		break;
 	}
@@ -870,6 +1006,9 @@ int __rpc_sockisbound(int fd)
 		struct sockaddr_in sin;
 		struct sockaddr_in6 sin6;
 		struct sockaddr_un usin;
+#ifdef RPC_VSOCK
+		struct sockaddr_vm svm;
+#endif /* VSOCK */
 	} u_addr;
 	socklen_t slen;
 
@@ -886,6 +1025,11 @@ int __rpc_sockisbound(int fd)
 		memcpy(&u_addr.sin6, &ss, sizeof(u_addr.sin6));
 		return (u_addr.sin6.sin6_port != 0);
 #endif
+#ifdef RPC_VSOCK
+	case AF_VSOCK:
+		memcpy(&u_addr.svm, &ss, sizeof(u_addr.svm));
+		return (u_addr.svm.svm_port != 0);
+#endif /* VSOCK */
 	case AF_LOCAL:
 		/* XXX check this */
 		memcpy(&u_addr.usin, &ss, sizeof(u_addr.usin));
@@ -907,9 +1051,6 @@ __rpc_set_netbuf(struct netbuf *nb, const void *ptr, size_t len)
 		if (nb->len)
 			mem_free(nb->buf, nb->len);
 		nb->buf = mem_zalloc(len);
-		if (nb->buf == NULL)
-			return NULL;
-
 		nb->maxlen = nb->len = len;
 	}
 	memcpy(nb->buf, ptr, len);
